/*
 * Copyright © 2016 by Data Virtuality GmbH
 */
package com.datavirtuality.dv.core.interfaces;

import org.apache.commons.lang3.text.StrLookup;
import org.apache.commons.lang3.text.StrMatcher;
import org.apache.commons.lang3.text.StrSubstitutor;

import java.util.HashMap;
import java.util.Map;
import java.util.Properties;

/**
 * Uniform properties holder.
 *
 * @author Roman Pedchenko
 * @date 2016.08.27
 */
public class DVPropertyHolder {
    public static class SingletonHolder {
        public static final DVPropertyHolder HOLDER_INSTANCE = new DVPropertyHolder();
    }

    public static DVPropertyHolder getInstance() {
        return SingletonHolder.HOLDER_INSTANCE;
    }

    private DVPropertyHolder() {
    }

    private final Map<String, Properties> map = new HashMap<>();

    public void add(String name, Properties props) {
        final StrSubstitutor sub = new StrSubstitutor(StrLookup.systemPropertiesLookup(), StrSubstitutor.DEFAULT_PREFIX,
                StrSubstitutor.DEFAULT_SUFFIX, StrSubstitutor.DEFAULT_ESCAPE, StrMatcher.stringMatcher(":"));
        props.forEach((k, v) -> {
            props.put(k, sub.replace(v));
        });
        map.put(name.toUpperCase(), props);
    }

    public Properties get(String name) {
        Properties result = map.get(name.toUpperCase());
        if (result == null)
            result = new Properties();
        return result;
    }

    private <T> T getQuartzProperty(String name) {
        final Properties props = get("quartz");
        return (T) props.get(name);
    }

    public int getQuartzInt(String name, int defVal) {
        return new QuartzPropertyGetter(this) {
            @Override
            Integer getValue(String propThrs) {
                return Integer.parseInt(propThrs);
            }
        }.getValue(name, defVal);
    }

    public boolean getQuartzBoolean(String name, boolean defVal) {
        return new QuartzPropertyGetter(this) {
            @Override
            Boolean getValue(String propThrs) {
                return Boolean.valueOf(propThrs);
            }
        }.getValue(name, defVal);
    }

    public String getQuartzString(String name, String defVal) {
        return new QuartzPropertyGetter(this) {
            @Override
            String getValue(String propThrs) {
                return propThrs;
            }
        }.getValue(name, defVal);
    }

    private abstract static class QuartzPropertyGetter {
        private DVPropertyHolder dvPropertyHolder;

        public QuartzPropertyGetter(DVPropertyHolder dvPropertyHolder) {
            this.dvPropertyHolder = dvPropertyHolder;
        }

        abstract <T> T getValue(String propThrs) throws Exception;

        public <T> T getValue(String name, T defVal) {
            String propThrs = dvPropertyHolder.getQuartzProperty(name);
            T value = defVal;
            if (propThrs != null)
                try {
                    value = getValue(propThrs);
                } catch (Exception e) {
                    return defVal;
                }
            return value;
        }
    }
}
