/*
 * Copyright © 2021 by Data Virtuality GmbH
 */
package com.datavirtuality.dv.core.interfaces;

import org.junit.After;
import org.junit.Test;

import java.util.Properties;

import static org.junit.Assert.*;

/**
 * @author Roman Pedchenko
 * @date 2021.09.24
 */
public class DVPropertyHolderTest {
    public static final String TEST = "com.dv.test";
    public static final String QUARTZ = "quartz";

    @After
    public void tearDown() throws Exception {
        System.clearProperty(TEST);
    }

    @Test
    public void propSubstitution() {
        System.setProperty(TEST, "val2");
        final DVPropertyHolder ph = DVPropertyHolder.getInstance();
        Properties p = new Properties();
        p.setProperty("com.dv.prop1", "val1");
        p.setProperty("com.dv.prop2", "${com.dv.test}");
        p.setProperty("com.dv.prop3", "${com.dv.test1:val3}");
        ph.add(TEST, p);
        final Properties p1 = ph.get(TEST);
        assertNotNull(p1);
        assertEquals("val1", p1.getProperty("com.dv.prop1"));
        assertEquals("val2", p1.getProperty("com.dv.prop2"));
        assertEquals("val3", p1.getProperty("com.dv.prop3"));
    }

    @Test
    public void getQuartzInt() {
        System.setProperty(TEST, "15");
        final DVPropertyHolder ph = DVPropertyHolder.getInstance();
        Properties p = new Properties();
        p.setProperty("com.dv.prop1", "${com.dv.test:10}");
        p.setProperty("com.dv.prop2", "${com.dv.test1:2}");
        p.setProperty("com.dv.prop3", "easy peasy lemon squeezy");
        ph.add(QUARTZ, p);
        assertEquals(15, ph.getQuartzInt("com.dv.prop1", -1));
        assertEquals(2, ph.getQuartzInt("com.dv.prop2", -1));
        assertEquals(-1, ph.getQuartzInt("com.dv.prop3", -1));
        assertEquals(-1, ph.getQuartzInt("com.dv.prop4", -1));
    }
}
